import 'package:flutter/material.dart';
import 'package:help_me/help_me.dart';

import 'help_me_clipper.dart';

class Help extends StatefulWidget {
  List<HelpMeItem> guides;
  Size? size;
  BuildContext pageContext;
  Widget? nextButton;
  Widget? skipButton;
  final Color? color;
  final Gradient? gradient;
  final Stream<HelpAction> helpActions;

  Help(
      {Key? key,
      required this.guides,
      required this.pageContext,
      this.nextButton,
      this.skipButton,
      this.color,
      required this.helpActions,
      this.gradient})
      : super(key: key) {
    size = Size(MediaQuery.of(pageContext).size.width,
        MediaQuery.of(pageContext).size.height);
  }

  @override
  State<Help> createState() => _HelpState();
}

class _HelpState extends State<Help> {
  int selected = 0;
  bool isShowing = true;
  RenderBox? currentBox;
  RenderBox? guideBox;
  Offset get currentPosition =>
      currentBox?.localToGlobal(Offset.zero) ?? Offset(0, 0);
  Offset get currentGuidePosition =>
      guideBox?.localToGlobal(Offset.zero) ?? Offset(0, 0);
  RenderBox? nextButtonBox;
  Alignment align = Alignment.bottomCenter;
  // RenderBox? guidBox;
  GlobalKey? skipButtonKey;
  var nextKey = GlobalKey();
  var oldSize;

  List<Widget> getItemsOnPage(bool reverse) {
    List<Widget> list = [];
    if (reverse) {
      list.add(_buttons());
      list.add(_guid());
    } else {
      list.add(_guid());
      list.add(_buttons());
    }

    return list;
  }

  _nextGuide() async {
    // guidBox = widget.guides[selected].guidKey.currentContext!.findRenderObject()
    //     as RenderBox;

    // await Future.delayed(Duration(milliseconds: 1000));
    if (widget.guides.length - 1 == selected) {
      _skip();
      return;
    }
    setState(() {
      selected++;
      currentBox = widget.guides[selected].key.currentContext!
          .findRenderObject() as RenderBox;
    });
  }

  _backGuide() async {
    // guidBox = widget.guides[selected].guidKey.currentContext!.findRenderObject()
    //     as RenderBox;

    // await Future.delayed(Duration(milliseconds: 1000));
    if (selected == 0) {
      return;
    }
    setState(() {
      selected--;
      currentBox = widget.guides[selected].key.currentContext!
          .findRenderObject() as RenderBox;
    });
  }

  _skip() {
    Navigator.of(context).pop();
  }

  _buttons() {
    if (widget.nextButton == null) {
      return Container();
    }

    var align = Alignment.bottomCenter;
    if (nextButtonBox != null) {
      var nextButtonPos = nextButtonBox!.localToGlobal(Offset.zero).dy;
      // var guidPos = guidBox!.localToGlobal(Offset.zero).dy;
      if (currentPosition.dy + (currentBox?.size.height ?? 0) >
          (nextButtonPos + ((widget.size!.height / 2) - nextButtonPos))) {
        align = Alignment.topCenter;
      }
    }

    return AnimatedAlign(
      duration: Duration(milliseconds: 200),
      alignment: align,
      child: Padding(
        padding: MediaQuery.of(widget.pageContext).padding,
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: [
            widget.skipButton != null
                ? GestureDetector(
                    child: widget.skipButton,
                    onTap: () {
                      _skip();
                    },
                  )
                : Container(),
            GestureDetector(
              child: widget.nextButton,
              onTap: () {
                _nextGuide();
              },
            ),
          ],
        ),
      ),
    );
  }

  _guid() {
    var buttonIsBottom = true;
    double buttonHeight = 0;
    double guideHeight = 0;

    double guideTopPadding = 0;
    double guideBottomPadding = 0;

    if (nextButtonBox != null) {
      buttonHeight = nextButtonBox!.size.height;
      guideHeight = currentBox!.size.height;
      if (currentPosition.dy + (currentBox?.size.height ?? 0) >
          (nextButtonBox!.localToGlobal(Offset.zero).dy +
              ((widget.size!.height / 2) -
                  nextButtonBox!.localToGlobal(Offset.zero).dy))) {
        buttonIsBottom = false;
      }
    }
    align = Alignment.bottomCenter;

    if (buttonIsBottom) {
      if (widget.size!.height -
              currentPosition.dy +
              (currentBox?.size.height ?? 0) -
              buttonHeight >
          currentPosition.dy) {
        guideBottomPadding = buttonHeight;
        align = Alignment.bottomCenter;
      } else {
        align = Alignment.topCenter;
      }
    } else {
      guideTopPadding = buttonHeight;
      align = Alignment.topCenter;
    }

    Future.delayed(Duration(milliseconds: 0)).then((value) {
      setState(() {
        guideBox = (widget.guides[selected].guideWidget.key as GlobalKey)
            .currentContext!
            .findRenderObject() as RenderBox;
      });
    });

    var bottom = (widget.size!.height -
            currentBox!.localToGlobal(Offset.zero).dy -
            (guideBox?.size.height ?? 0) -
            currentBox!.size.height) -
        70;
    if (bottom < 0) {
      bottom = 24;
    }
    var top =
        ((widget.size!.height - (currentBox!.localToGlobal(Offset.zero).dy))) +
            70;
    print("BOTTTOM:$bottom");
    print("AASDDD:${currentBox!.localToGlobal(Offset.zero).dy}");

    return AnimatedPositioned(
      duration: Duration(milliseconds: 200),
      right: 0,
      left: 0,
      bottom: align == Alignment.bottomCenter ? bottom : top,
      child: AnimatedSwitcher(
        duration: Duration(milliseconds: 200),
        child: isShowing ? widget.guides[selected].guideWidget : Container(),
      ),
    );
  }

  @override
  void initState() {
    currentBox = widget.guides[selected].key.currentContext!.findRenderObject()
        as RenderBox;
    if (widget.nextButton != null) {
      widget.nextButton = Container(
        child: widget.nextButton,
        key: nextKey,
      );
    }
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      guideBox = (widget.guides[selected].guideWidget.key as GlobalKey)
          .currentContext!
          .findRenderObject() as RenderBox;
    });
    Future.delayed(Duration(milliseconds: 200), () => postFrameCallback());
    // WidgetsBinding.instance?.addPostFrameCallback((_) => postFrameCallback());
    widget.helpActions.listen((event) {
      if (event == HelpAction.NEXT) {
        _nextGuide();
      } else if (event == HelpAction.SKIP) {
        _skip();
      } else if (event == HelpAction.BACK) {
        _backGuide();
      }
    });
    super.initState();
  }

  void postFrameCallback() {
    var context = nextKey.currentContext;
    if (context == null) return;
    var newSize = context.findRenderObject() as RenderBox;

    setState(() {
      nextButtonBox = newSize;
      // guidBox = newSizeGuid;
      var nextButtonPos = nextButtonBox!.localToGlobal(Offset.zero).dy;
      // var guidPos = guidBox!.localToGlobal(Offset.zero).dy;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      height: double.infinity,
      child: Stack(
        children: [
          AnimatedSwitcher(
            duration: Duration(milliseconds: 200),
            child: Builder(builder: (context) {
              if (widget.size != null) {
                if (currentPosition.dy < 0 ||
                    currentPosition.dy > widget.size!.height) {
                  isShowing = false;
                  _moveScroll(currentBox!);
                  // widget.scrollController!.jumpTo(
                  //   position.dy,
                  // );

                }
              }

              return isShowing
                  ? Container(
                      child: AnimatedSwitcher(
                        duration: Duration(milliseconds: 300),
                        child: CustomPaint(
                          key: ValueKey(widget.guides[selected].key.hashCode),
                          painter: HelpMePainter(widget.guides[selected].key,
                              currentBox!, guideBox, align,
                              arrow: widget.guides[selected].arrow,
                              shape: widget.guides[selected].shape),
                          child: ClipPath(
                            clipper: HelpMeClipper(
                                widget.guides[selected].key, currentBox!,
                                shape: widget.guides[selected].shape),
                            key: ValueKey(
                                widget.guides[selected].key.hashCode + 1),
                            child: Container(
                              decoration: BoxDecoration(
                                  color: widget.gradient == null
                                      ? (widget.color ??
                                          Colors.black.withOpacity(0.01))
                                      : null,
                                  gradient: widget.gradient),
                            ),
                          ),
                        ),
                      ),
                    )
                  : Container();
            }),
          ),
          // if (currentPosition.dy < 0 ||
          //     currentPosition.dy > widget.size!.height)
          //   Container()
          // else
          _buttons(),
          _guid()
        ],
      ),
    );
  }

  _moveScroll(
    RenderBox box,
  ) async {
    // Offset position = box.localToGlobal(Offset.zero);
    // double scroll = 0;
    // if (position.dy > widget.size!.height) {
    //   scroll = position.dy -
    //       widget.scrollController!.offset +
    //       box.paintBounds.height;
    // } else {
    //   scroll = position.dy + widget.scrollController!.offset;
    // }
    // print(scroll);
    // widget.scrollController!.jumpTo(scroll);
    Scrollable.ensureVisible(widget.guides[selected].key.currentContext!,
        duration: Duration(milliseconds: 200), curve: Curves.easeIn);
    // widget.scrollController!.animateTo(
    //   scroll,
    //   duration: Duration(milliseconds: 200),
    //   curve: Curves.easeIn,
    // );
    await Future.delayed(Duration(milliseconds: 600));
    setState(() {
      isShowing = true;
    });
  }
}
