import 'package:flutter/material.dart';

import 'guide.dart';
import 'dart:math' as math;

class HelpMePainter extends CustomPainter {
  GlobalKey key;
  RenderBox box;
  RenderBox? guide;
  Alignment align;
  HelpMeShape shape;
  Arrow? arrow;
  HelpMePainter(this.key, this.box, this.guide, this.align,
      {required this.shape, this.arrow});
  @override
  void paint(Canvas canvas, Size size) {
    // Path path0 = Path();

    // var size2 = Size(300, 300);

    // var bound = box;
    var size2 = box.size;

    Path path0;
    if (shape is HelpMeCustomShape) {
      path0 = (shape as HelpMeCustomShape).path;
    } else if (shape is HelpMeOvalShape) {
      path0 = Path()
        ..addOval(Rect.fromLTWH(box.paintBounds.left, box.paintBounds.top,
            box.paintBounds.width, box.paintBounds.height));
    } else if (shape is HelpMeRectShape) {
      path0 = Path()
        ..addRRect(RRect.fromRectAndCorners(box.paintBounds,
            bottomLeft: (shape as HelpMeRectShape).borderRadius.bottomLeft,
            bottomRight: (shape as HelpMeRectShape).borderRadius.bottomRight,
            topLeft: (shape as HelpMeRectShape).borderRadius.topLeft,
            topRight: (shape as HelpMeRectShape).borderRadius.topRight));
    } else {
      path0 = Path()..addRRect(RRect.fromRectAndCorners(box.paintBounds));
    }
    var path = Path()
      ..addRect(Rect.fromLTWH(0, 0, size.width, size.height))
      ..addPath(path0, box.localToGlobal(Offset.zero))
      // ..addOval(Rect.fromCircle(center: position, radius: 40))
      ..fillType = PathFillType.evenOdd;
    var paint = Paint();
    paint.color = Colors.black87;

    paint.style = PaintingStyle.fill;
    canvas.drawPath(path, paint);
    var path2 = Path()..addPath(path0, box.localToGlobal(Offset.zero));
    var paint2 = Paint()..color = shape.color;
    paint2.style = PaintingStyle.stroke;
    paint2.strokeWidth = 5;
    if (arrow != null) {
      var isCurved = (box.size.width < (size.width / 2)) &&
          box.localToGlobal(Offset.zero).dx != size.width / 2;
      var isRight =
          box.localToGlobal(Offset.zero).dx > size.width / 2 ? true : false;
      _arrow(canvas,
          arrow: arrow!,
          isCurved: isCurved,
          isRight: isCurved == false ? true : isRight,
          destination: Offset(
              box.localToGlobal(Offset.zero).dx +
                  (isCurved
                      ? (isRight ? 0 : box.size.width)
                      : (box.size.width / 2)),
              box.localToGlobal(Offset.zero).dy +
                  (align == Alignment.bottomCenter
                      ? ((isCurved) ? box.size.height / 2 : box.size.height)
                      : 0)),
          origin: Offset(
              size.width / 2,
              ((guide?.localToGlobal(Offset.zero).dy) ?? 0) +
                  (align == Alignment.topCenter
                      ? (guide?.size.height ?? 0)
                      : 0)));
    }
    canvas.drawPath(path2, paint2);
  }

  _arrow(
    Canvas canvas, {
    required bool isRight,
    required Offset destination,
    required Offset origin,
    required bool isCurved,
    required Arrow arrow,
  }) {
    Paint paint = Paint();
    paint.color = arrow.color;
    paint.strokeWidth = arrow.width;
    paint.style = PaintingStyle.stroke;
    Path path = Path();

    var middlePoint;

    if (isRight) {
      middlePoint = Offset(
          (origin.dx + destination.dx - (origin - destination).dy) / 2,
          (origin.dy + destination.dy + (origin - destination).dx) / 2);
    } else {
      middlePoint = Offset(
          (origin.dx + destination.dx + (origin - destination).dy) / 2,
          (origin.dy + destination.dy - (origin - destination).dx) / 2);
    }
    path.moveTo(origin.dx, origin.dy);
    if (isCurved) {
      path.quadraticBezierTo(
          middlePoint.dx, middlePoint.dy, destination.dx, destination.dy);
    } else {
      path.lineTo(destination.dx, destination.dy);
    }
    var arrowPoint1;
    if (isCurved) {
      var d = (destination - origin).distance;
      int arrowMetric = 15;
      var t = (d - arrowMetric) / d;
      arrowPoint1 = Offset(((1 - t) * origin.dx + t * destination.dx),
          ((1 - t) * origin.dy + t * destination.dy));
    } else {
      var d = (destination - middlePoint).distance;
      int arrowMetric = 15;
      var t = (d - arrowMetric) / d;
      arrowPoint1 = Offset(((1 - t) * middlePoint.dx + t * destination.dx),
          ((1 - t) * middlePoint.dy + t * destination.dy));
    }
    path.lineTo(arrowPoint1.dx, arrowPoint1.dy);
    path.moveTo(destination.dx, destination.dy);

    var a;
    var b;
    var c;
    if (isCurved) {
      a = destination.dy - middlePoint.dy;
      b = (middlePoint.dx - destination.dx);
      c = (destination.dx * middlePoint.dy - middlePoint.dx * destination.dy);
    } else {
      a = destination.dy - origin.dy;
      b = (origin.dx - destination.dx);
      c = (destination.dx * origin.dy - origin.dx * destination.dy);
      isRight = false;
    }
    var distance = ((a * arrowPoint1.dx + b * arrowPoint1.dy + c).abs());
    var arrowPoint2Dx;
    var arrowPoint2Dy;
    if (isRight) {
      arrowPoint2Dx = arrowPoint1.dx +
          ((2 * distance) / (math.pow(a, 2) + math.pow(b, 2))) * a;
      arrowPoint2Dy = arrowPoint1.dy +
          ((2 * distance) / (math.pow(a, 2) + math.pow(b, 2))) * b;
    } else {
      arrowPoint2Dx = arrowPoint1.dx -
          ((2 * distance) / (math.pow(a, 2) + math.pow(b, 2))) * a;
      arrowPoint2Dy = arrowPoint1.dy -
          ((2 * distance) / (math.pow(a, 2) + math.pow(b, 2))) * b;
    }

    path.lineTo(arrowPoint2Dx, arrowPoint2Dy);
    canvas.drawPath(path, paint);

    // path = createAnimatedPath(path, animationPercent);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}

class HelpMeClipper extends CustomClipper<Path> {
  GlobalKey key;
  RenderBox box;

  HelpMeShape shape;
  HelpMeClipper(this.key, this.box, {required this.shape});
  @override
  Path getClip(Size size) {
    var size2 = box.size;

    Path path0;
    if (shape is HelpMeCustomShape) {
      path0 = (shape as HelpMeCustomShape).path;
    } else if (shape is HelpMeOvalShape) {
      path0 = Path()
        ..addOval(Rect.fromLTWH(box.paintBounds.left, box.paintBounds.top,
            box.paintBounds.width, box.paintBounds.height));
    } else if (shape is HelpMeRectShape) {
      path0 = Path()
        ..addRRect(RRect.fromRectAndCorners(box.paintBounds,
            bottomLeft: (shape as HelpMeRectShape).borderRadius.bottomLeft,
            bottomRight: (shape as HelpMeRectShape).borderRadius.bottomRight,
            topLeft: (shape as HelpMeRectShape).borderRadius.topLeft,
            topRight: (shape as HelpMeRectShape).borderRadius.topRight));
    } else {
      path0 = Path()..addRRect(RRect.fromRectAndCorners(box.paintBounds));
    }
    var path = Path()
      ..addRect(Rect.fromLTWH(0, 0, size.width, size.height))
      ..addPath(path0, box.localToGlobal(Offset.zero))
      // ..addOval(Rect.fromCircle(center: position, radius: 40))
      ..fillType = PathFillType.evenOdd;
    return path;
  }

  @override
  bool shouldReclip(covariant CustomClipper<Path> oldClipper) {
    return true;
  }
}
